/*
    BFilter - a smart ad-filtering web proxy
    Copyright (C) 2002-2007  Joseph Artsimovich <joseph_a@mail.ru>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef CLIENTREQUESTDESCRIPTOR_H_
#define CLIENTREQUESTDESCRIPTOR_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "RefCountable.h"
#include "RefCounter.h"
#include <ace/config-lite.h>
#include <ace/Synch_Traits.h>
#include <sigc++/sigc++.h>
#include <gtkmm/treeiter.h>
#include <glibmm/ustring.h>

namespace GtkGUI
{

class DebugMessage;

class ClientRequestDescriptor
:	public RefCountable<RefCounter<ACE_NULL_SYNCH> >,
	public sigc::trackable
{
	DECLARE_NON_COPYABLE(ClientRequestDescriptor)
public:
	typedef sigc::slot1<void, bool> SelectionChangeFunctor;
	typedef sigc::slot2<Gtk::TreeIter, Glib::ustring const&,
		SelectionChangeFunctor const&> ListAppendFunctor;
	typedef sigc::slot1<void, Gtk::TreeIter> ListRemoveFunctor;
	typedef sigc::slot1<void, Gtk::TreeIter> ListSelectFunctor;
	
	ClientRequestDescriptor(
		Glib::ustring const& url,
		ListAppendFunctor const& list_append_functor,
		ListRemoveFunctor const& list_remove_functor,
		ListSelectFunctor const& list_select_functor);
	
	virtual ~ClientRequestDescriptor();
	
	void registerMessage(DebugMessage* msg);
	
	void unregisterMessage(DebugMessage* msg);
	
	void onMessageSelected();
private:
	void onListSelectionChange(bool selected);
	
	Glib::ustring m_url;
	ListAppendFunctor m_listAppendFunctor;
	ListRemoveFunctor m_listRemoveFunctor;
	ListSelectFunctor m_listSelectFunctor;
	Gtk::TreeIter m_requestListIter;
	DebugMessage* m_pFirstMessage;
	DebugMessage* m_pLastMessage;
};

} // namespace GtkGUI

#endif
